package com.yixin.ms.server;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.math.MathUtil;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.crypto.util.Sm4Util;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.dto.ThirdPlatformDTO;
import com.yixin.ms.model.dto.ThirdPlatformRespDto;
import com.yixin.ms.model.dto.ThirdPlatformSm4DTO;
import com.yixin.ms.model.dto.ThirdPlatformSm4RespDto;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.ChooseVO;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.model.vo.MultipleChooseItemVO;
import com.yixin.ms.model.vo.MultipleChooseVO;
import com.yixin.ms.server.dto.QuTongIdentityAuthDto;
import com.yixin.ms.server.dto.ScenicVerifyDTO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.QuTongIdentityVO;
import com.yixin.ms.server.vo.QuTongIdentityVO.RespCodeEnum;
import com.yixin.ms.server.vo.ScenicVerifyVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @description:
 * @ClassName HealthOpenReturnServer
 * @Author WXG
 * @Date 2022/10/20 17:34
 */
@Slf4j
@Service
public class HealthOpenReturnServer {
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;

    //默认广告地址
    private static String defaultAdvertise = "https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/ad.png";
    //通用模式
    private static String SHOW_PAGE_TYPE_COMMON = "common";
    //景区模式
    private static String SHOW_PAGE_TYPE_SCENIC = "scenic";
    //状态
    private static String STATUS_LOADING = "loading";
    private static String STATUS_PENDING = "pending";
    private static String STATUS_PENDING_MULTIPLE = "pendingMultiple";
    private static String STATUS_CLOSING = "closing";

    public void healthOpenReturn(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        userVO.setShowPageType("scenic");
        //开门声音提示
        String openMsg = placeInfoVO.getOpenMsg();
        String openVoiceMsg = "";
        userVO.setIsOpen(placeInfoVO.getIsOpen());

        /*******************************以下为三方平台逻辑*******************************/
        //TODO...联动平台三方接口 三方接口配置参数优先与系统配置参数
        handleThirdPlatform(userVO, placeInfoVO, shortTermDTO);
        /*******************************以下为通行结果逻辑*******************************/
        //通行结果转换
        String subLine = openMsg;
        if (userVO.getIsOpen() != null
                && userVO.getIsOpen().intValue() == HealthOpenEnum.NO_THOROUGHFARE.getCode().intValue()) {//禁止通行
            //是否开启警报
            userVO.setVaccineIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/warning.png");
            userVO.setVaccineNumColor("#ed593b");
            openMsg = Preconditions.isNotBlank(openMsg) ? openMsg : "禁止通行";
            subLine = Preconditions.isNotBlank(subLine) ? subLine : "禁止通行";

            userVO.setOpenTimes(0);
            userVO.setAMainIcon(Preconditions.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.AFORBID.getUrl());
            userVO.setAMainLine(Preconditions.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : openMsg);
        } else {//请通行
            userVO.setAMainIcon(Preconditions.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.APASS.getUrl());
            userVO.setAMainLine(Preconditions.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : openMsg);
        }

        //设置开门信号提示信息
        openVoiceMsg = openMsg;
        userVO.setOpenMsg(Preconditions.isNotBlank(subLine) ? subLine : openMsg);
        userVO.setOpenVoiceMsg(openVoiceMsg);
        //设置通行结果数据
        HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
        if (Preconditions.isNotBlank(codeTypeVO)) {
            codeTypeVO.setContent(subLine);
            userVO.setData(Arrays.asList(codeTypeVO));
        }
    }

    /**
     * 三方接入平台
     *
     * @param userVO
     * @param placeInfoVO
     * @param shortTermDTO
     */
    private void handleThirdPlatform(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        String thirdOpenMsg = null;
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0)
                && Preconditions.isNotBlank(placeInfoVO.getOpenThirdLinkPlatform())
                && placeInfoVO.getOpenThirdLinkPlatform()
                && Preconditions.isNotBlank(placeInfoVO.getAppKey())
                && Preconditions.isNotBlank(placeInfoVO.getAppSecret())
                && Preconditions.isNotBlank(placeInfoVO.getPrivateKey())) {
            if (Preconditions.isNotBlank(placeInfoVO.getInterfaceAddress())) {
                try {
                    ThirdPlatformDTO dto = new ThirdPlatformDTO();
                    if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {//二维码
                        dto.setVerifyType("QRCODE");
                        userVO.setVaccineNum("二维码认证");
                    }else {//身份证
                        dto.setVerifyType("IDCARD");
                        userVO.setVaccineNum("身份证认证");
                    }
                    dto.setSn(shortTermDTO.getSn());
                    dto.setIdNumber(userVO.getIdNumber());
                    dto.setName(userVO.getUsername());
                    dto.setPhone(shortTermDTO.getPhone());
                    dto.setQrcode(shortTermDTO.getHealthCode());

                    ThirdPlatformSm4DTO sm4Dto = new ThirdPlatformSm4DTO();
                    long currentTimeMillis = System.currentTimeMillis();
                    sm4Dto.setReqTimestamp(currentTimeMillis);
                    sm4Dto.setSign(Sm4Util.getSign(placeInfoVO.getAppKey(), placeInfoVO.getAppSecret(), String.valueOf(currentTimeMillis)));
                    sm4Dto.setBizContent(Sm4Util.encrypt(placeInfoVO.getPrivateKey(), JSONObject.toJSONString(dto)));
                    log.info("联动平台三方接口调用加密前入参信息：{}", JSON.toJSONString(dto));
                    long startTime = System.currentTimeMillis();
                    String body = HttpUtil.post(placeInfoVO.getInterfaceAddress(), JSONObject.toJSONString(sm4Dto), 5000);
                    log.info("----->三方响应耗时:{}", System.currentTimeMillis() - startTime);
                    RestResult result = JSON.parseObject(body, RestResult.class);
                    log.info("联动平台三方接口响应原始信息：{}", JSON.toJSONString(result));
                    if (Preconditions.isNotBlank(result)
                            && result.getCode() == RestCodeEnum.SUCCESS.getCode()
                            && Preconditions.isNotBlank(result.getData())) {
                        ThirdPlatformSm4RespDto thirdPlatformSm4RespDto = JSON.parseObject(JSON.toJSONString(result.getData()), ThirdPlatformSm4RespDto.class);
                        //验证签名
                        Boolean checkSign = Sm4Util.checkSign(placeInfoVO.getAppKey(), placeInfoVO.getAppSecret(), thirdPlatformSm4RespDto.getSign(), String.valueOf(thirdPlatformSm4RespDto.getReqTimestamp()));
                        if (checkSign) {
                            String decrypt = Sm4Util.decrypt(placeInfoVO.getPrivateKey(), thirdPlatformSm4RespDto.getBizContent());
                            log.info("联动平台三方接口响应内容解密：{}", decrypt);
                            ThirdPlatformRespDto thirdPlatformRespDto = JSON.parseObject(decrypt, ThirdPlatformRespDto.class);
                            convertThirdOpenReturn(userVO, thirdPlatformRespDto, placeInfoVO);
                            return;
                        } else {
                            thirdOpenMsg = "联动平台三方接口返回签名错误";
                        }
                    } else {
                        thirdOpenMsg = Preconditions.isNotBlank(result.getMsg()) ? result.getMsg() : null;
                    }
                } catch (Exception e) {
                    log.error("联动平台三方接口响应异常：{}", e.getMessage());
                    e.printStackTrace();
                    thirdOpenMsg = "联动平台三方接口响应异常";
                }
            } else {
                thirdOpenMsg = "联动平台三方接口未配置\n请联系管理员";
            }
            throw new ServiceException(Preconditions.isNotBlank(thirdOpenMsg) ? thirdOpenMsg : "禁止通行");
        }
    }

    //处理联动平台三方接口返回参数
    private void convertThirdOpenReturn(HealtCodeUserVO userVO, ThirdPlatformRespDto thirdPlatformRespDto, PlaceInfoVO placeInfoVO) {

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getShowPageType())
                && !SHOW_PAGE_TYPE_SCENIC.equals(thirdPlatformRespDto.getShowPageType())
                && !SHOW_PAGE_TYPE_COMMON.equals(thirdPlatformRespDto.getShowPageType())) {
            throw new ServiceException("联动平台三方返回内容有误");
        }

        //单次模式 通用模版
        if (Preconditions.isBlank(thirdPlatformRespDto.getShowPageType()) || SHOW_PAGE_TYPE_COMMON.equals(thirdPlatformRespDto.getShowPageType())) {
            convertCommonThirdOpenReturn(userVO, thirdPlatformRespDto, placeInfoVO);
            return;
        }

        //判断是否是景区连续模式 景区模版
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus())
                && Preconditions.isNotBlank(thirdPlatformRespDto.getShowPageType())
                && SHOW_PAGE_TYPE_SCENIC.equals(thirdPlatformRespDto.getShowPageType())) {
            convertScenicThirdOpenReturn(userVO, thirdPlatformRespDto, placeInfoVO);
            return;
        }

    }

    private void convertScenicThirdOpenReturn(HealtCodeUserVO userVO, ThirdPlatformRespDto thirdPlatformRespDto, PlaceInfoVO placeInfoVO) {
        userVO.setShowPageType(thirdPlatformRespDto.getShowPageType());
        userVO.setStatus(thirdPlatformRespDto.getStatus());
        userVO.setTaskId(thirdPlatformRespDto.getTaskId());

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getEnd())) {
            userVO.setEnd(thirdPlatformRespDto.getEnd());
        }

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getDelay())) {
            userVO.setDelay(thirdPlatformRespDto.getDelay());
        }

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getIsOpen())) {
            userVO.setIsOpen(thirdPlatformRespDto.getIsOpen());
        }
        if (userVO.getIsOpen().intValue() == 1) {
            userVO.setOpenTimes(0);
        }

        //loading状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && STATUS_LOADING.equals(thirdPlatformRespDto.getStatus())) {
            userVO.setDBackgroundUrl(thirdPlatformRespDto.getDBackgroundUrl());
            userVO.setDMainLine(thirdPlatformRespDto.getDMainLine());
            //开门信号格式
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalType())) {
                userVO.setUseSerialPort(thirdPlatformRespDto.getSignalType());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getOpenTimes())) {
                userVO.setOpenTimes(thirdPlatformRespDto.getOpenTimes());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalNo())) {
                userVO.setSignalNo(thirdPlatformRespDto.getSignalNo());
            }

            return;
        }

        //pending状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && STATUS_PENDING.equals(thirdPlatformRespDto.getStatus())) {
            if (Preconditions.isBlank(thirdPlatformRespDto.getTaskId()) || Preconditions.isBlank(thirdPlatformRespDto.getBChoiseList())) {
                throw new ServiceException("PENDING状态下三方平台接口响应参数有误");
            }

            userVO.setBBackgroundUrl(thirdPlatformRespDto.getBBackgroundUrl());
            //设置默认图
            String icon = ThirdPlatformPassIconEnum.iconOf(thirdPlatformRespDto.getBMainIcon(), Boolean.FALSE);
            userVO.setBMainIcon(Preconditions.isBlank(icon) ? null : icon);
            userVO.setBSubLine(thirdPlatformRespDto.getBSubLine());
            userVO.setBMainLine(thirdPlatformRespDto.getBMainLine());
            userVO.setBChoiseList(thirdPlatformRespDto.getBChoiseList());
            return;
        }

        //pendingMultiple状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && STATUS_PENDING_MULTIPLE.equals(thirdPlatformRespDto.getStatus())) {
            if (Preconditions.isBlank(thirdPlatformRespDto.getTaskId()) || Preconditions.isBlank(thirdPlatformRespDto.getCChoiseList())) {
                throw new ServiceException("PENDING_MULTIPLE状态下三方平台接口响应参数有误");
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getCButtonText())) {
                userVO.setCButtonText(thirdPlatformRespDto.getCButtonText());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getCButtonSubText())) {
                userVO.setCButtonSubText(thirdPlatformRespDto.getCButtonSubText());
            }
            userVO.setCBackgroundUrl(thirdPlatformRespDto.getCBackgroundUrl());
            userVO.setCChoiseList(thirdPlatformRespDto.getCChoiseList());
            return;
        }

        //closing状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && STATUS_CLOSING.equals(thirdPlatformRespDto.getStatus())) {
            userVO.setABackgroundUrl(thirdPlatformRespDto.getABackgroundUrl());
            //头部
            if (Preconditions.isNotBlank(thirdPlatformRespDto)
                    && Preconditions.isNotBlank(thirdPlatformRespDto.getAMainLine())) {
                userVO.setAMainLine(thirdPlatformRespDto.getAMainLine());
            }
            String icon = ThirdPlatformPassIconEnum.iconOf(thirdPlatformRespDto.getAMainIcon(), Boolean.TRUE);
            userVO.setAMainIcon(Preconditions.isBlank(icon) ? null : icon);
            //通行结果
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getAMainIcon())
                    && thirdPlatformRespDto.getAMainIcon().intValue() == ThirdPlatformPassIconEnum.APASS.getIconCode()) {
                userVO.setAMainLine(thirdPlatformRespDto.getAMainLine());
            } else {
                userVO.setAMainLine(thirdPlatformRespDto.getAMainLine());
            }
            userVO.setASubLineFirst(thirdPlatformRespDto.getASubLineFirst());
            userVO.setASubLineSecond(thirdPlatformRespDto.getASubLineSecond());
            try {
                if (SMSOrIdCardUtils.isIdCard(thirdPlatformRespDto.getASubLineSecond())) {
                    userVO.setShowIdNumber(thirdPlatformRespDto.getASubLineSecond());
                    userVO.setASubLineSecond(DesensitizedUtil.idCardNum(thirdPlatformRespDto.getASubLineSecond(), 4, 4));
                }
            } catch (Exception e) {
                log.error("--->身份证校验有误");
            }
            userVO.setALineOneLabel(thirdPlatformRespDto.getALineOneLabel());
            userVO.setALineOneValue(thirdPlatformRespDto.getALineOneValue());
            userVO.setALineTwoLabel(thirdPlatformRespDto.getALineTwoLabel());
            userVO.setALineTwoValue(thirdPlatformRespDto.getALineTwoValue());
            userVO.setALineThrLabel(thirdPlatformRespDto.getALineThrLabel());
            //图片转换
            String aLineThrValue = thirdPlatformRespDto.getALineThrValue();
            userVO.setALineThrValue(aLineThrValue);
            //底部
            userVO.setQrcodeText(Preconditions.isNotBlank(thirdPlatformRespDto.getQrcodeText()) ? thirdPlatformRespDto.getQrcodeText() : userVO.getQrcodeText());
            userVO.setAdvertise(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? thirdPlatformRespDto.getAdvertise() : userVO.getAdvertise());
            userVO.setShowQrcode(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? Boolean.TRUE : Boolean.FALSE);
            //开门信号格式
            userVO.setIsOpen(Preconditions.isNotBlank(thirdPlatformRespDto.getIsOpen()) ? thirdPlatformRespDto.getIsOpen() : userVO.getIsOpen());
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalType())) {
                userVO.setUseSerialPort(thirdPlatformRespDto.getSignalType());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getOpenTimes())) {
                userVO.setOpenTimes(thirdPlatformRespDto.getOpenTimes());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalNo())) {
                userVO.setSignalNo(thirdPlatformRespDto.getSignalNo());
            }
            return;
        }

    }

    private void convertCommonThirdOpenReturn(HealtCodeUserVO userVO, ThirdPlatformRespDto thirdPlatformRespDto, PlaceInfoVO placeInfoVO) {

        //头部
        if (Preconditions.isNotBlank(thirdPlatformRespDto)
                && Preconditions.isNotBlank(thirdPlatformRespDto.getMainLine())) {
            userVO.setTitle(thirdPlatformRespDto.getMainLine());
        }
        //中下左部
        HealtCodeNucleinVO v = userVO.getNuclein().get(0);
        v.setTitle(Preconditions.isNotBlank(thirdPlatformRespDto.getMainLineLeft()) ? thirdPlatformRespDto.getMainLineLeft() : v.getTitle());
        v.setContent(Preconditions.isNotBlank(thirdPlatformRespDto.getSubLineLeft()) ? thirdPlatformRespDto.getSubLineLeft() : v.getContent());
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle(Preconditions.isNotBlank(thirdPlatformRespDto.getMainLineRight()) ? thirdPlatformRespDto.getMainLineRight() : userVO.getVaccineIcoTitle());
        userVO.setVaccineNum(Preconditions.isNotBlank(thirdPlatformRespDto.getSubLineRight()) ? thirdPlatformRespDto.getSubLineRight() : userVO.getVaccineNum());
        //底部
        userVO.setQrcodeText(Preconditions.isNotBlank(thirdPlatformRespDto.getQrcodeText()) ? thirdPlatformRespDto.getQrcodeText() : userVO.getQrcodeText());
        userVO.setAdvertise(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? thirdPlatformRespDto.getAdvertise() : userVO.getAdvertise());
        userVO.setShowQrcode(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? Boolean.TRUE : Boolean.FALSE);

        userVO.setIsOpen(Preconditions.isNotBlank(thirdPlatformRespDto.getIsOpen()) ? thirdPlatformRespDto.getIsOpen() : userVO.getIsOpen());
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalType())) {
            userVO.setUseSerialPort(thirdPlatformRespDto.getSignalType());
        }
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getOpenTimes())) {
            userVO.setOpenTimes(thirdPlatformRespDto.getOpenTimes());
        }
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalNo())) {
            userVO.setSignalNo(thirdPlatformRespDto.getSignalNo());
        }
    }
}
