package com.yixin.ms.server;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.symmetric.SM4;
import cn.hutool.crypto.symmetric.SymmetricCrypto;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.ClassUtil;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.crypto.util.SzhzSm2Util;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.HealthOpenEnum;
import com.yixin.ms.model.enums.IDTypeEnum;
import com.yixin.ms.model.enums.RestCodeEnum;
import com.yixin.ms.server.dto.*;
import com.yixin.ms.server.vo.DeviceScanIdCardVO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.QueryExamUserWhitelistVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import static com.yixin.ms.model.enums.PlaceAreaEnum.getByPlaceAreaCode;

/**
 * @author: lileilei
 * @date: 2022/6/23 10:47
 * @description: 杭州“四码一屏”统一接口
 */
@Component
@Slf4j
public class PlaceHealthCodeServer {

    @Value("${health.code.queryPlaceInfoBySnUrl}")
    private String queryPlaceInfoBySnUrl;
    /**
     * 通过设备sn查询账户信息
     */
    public PlaceInfoVO queryAdminInfoBySn(String sn,String idNumber) {
        PlaceInfoVO placeInfoVO = null;
        if (StrUtil.isNotBlank(sn) && StrUtil.isNotBlank(idNumber)) {
            DeviceReq deviceReq = new DeviceReq().setSn(sn).setIdNumber(idNumber);
            HttpResponse response = HttpRequest.post(queryPlaceInfoBySnUrl)
                    .body(JSON.toJSONString(deviceReq)).timeout(10000).execute();
            String responseBody = response.body();
            RestResult<PlaceInfoVO> result = JSON.parseObject(responseBody, new TypeReference<RestResult<PlaceInfoVO>>() {});
            log.info("通过设备sn: {}, 查询账户信息返回内容:{}", sn, JSON.toJSONString(result));
            ClassUtil.throwBusinessException(RestCodeEnum.SUCCESS.getCode() != result.getCode(), StrUtil.isNotBlank(result.getMsg()) ? result.getMsg() : "请求有误");
            placeInfoVO = result.getData();
        }
        ClassUtil.throwBusinessException(ObjectUtil.isNull(placeInfoVO),RestCode.DEVICE_NOT_FOUND);
        return placeInfoVO;
    }
}
