package com.yixin.ms.service.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.*;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.ClassUtil;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.*;
import com.yixin.ms.server.vo.*;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {

    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Resource
    private HealthOpenReturnServer healthOpenReturnServer;
    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }
    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult<HealtCodeUserVO> getShortTerm(ShortTermDTO shortTermDTO) {
        long starTime = System.currentTimeMillis();
        log.info("--->联动服务查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        ClassUtil.throwBusinessException(StrUtil.isBlank(shortTermDTO.getToken()), RestCode.FORBIDDEN);
        ClassUtil.throwBusinessException(StrUtil.isBlank(shortTermDTO.getIdNumber()), "身份证不能为空");
        ClassUtil.throwBusinessException(StrUtil.isBlank(shortTermDTO.getSn()), "设备号不能为空");
        /*查询设备账户信息 以及设备所绑账号的开关门信息*/
        PlaceInfoVO placeInfoVO = placeHealthCodeServer.queryAdminInfoBySn(shortTermDTO.getSn(),shortTermDTO.getIdNumber());
        //初始化参数
        HealtCodeUserVO userVO = initHealthUserVo(shortTermDTO, placeInfoVO);

        // 处理姓名、身份证、手机号数据
        convertUserInfo(userVO, shortTermDTO);

        // 业务及用户信息、开关门校验及语音提示 （三方平台接口、黑名单、衢通码等）
        resultUserVOInfo(userVO, placeInfoVO, shortTermDTO);

        log.info("开门信号结果返回:{}", JSON.toJSONString(userVO));
        log.info("----->总耗时：{}", System.currentTimeMillis() - starTime);
        return RestResponse.ok(userVO);
    }


    /**
     * 补全返回参数数据
     * 1：用户信息
     * 2：开关门规则校验 及语音提示
     *
     * @param userVO
     * @param placeInfoVO
     * @param shortTermDTO
     */
    private void resultUserVOInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        //开关门规则校验 及语音提示
        healthOpenReturnServer.healthOpenReturn(userVO, placeInfoVO, shortTermDTO);
    }

    /**
     * 处理用户信息 身份证 姓名 手机号
     *
     * @param userVO
     * @param shortTermDTO
     */
    private void convertUserInfo(HealtCodeUserVO userVO, ShortTermDTO shortTermDTO) {
        //姓名和身份证号脱敏处理
        userVO.setUsername(Preconditions.isBlank(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
        userVO.setIdNumber(Preconditions.isBlank(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());
        userVO.setShowIdNumber(Preconditions.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setShowUsername(Preconditions.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setPhone(Preconditions.isBlank(userVO.getPhone()) ? shortTermDTO.getPhone() : userVO.getPhone());
        userVO.setASubLineFirst(Preconditions.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setASubLineSecond(Preconditions.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
    }

    private HealtCodeUserVO initHealthUserVo(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        //验证方式
        String vaccineNum = "身份证验证";
        if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {
            vaccineNum = "二维码";
        }
        //全局背景色
        userVO.setBackgroundColor(ColorEnum.BLUE.getValue());
        //头部
        String title = "温馨提示";
        userVO.setTitle(title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());
        //中上部
        userVO.setVaccineIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setVaccineNumColor("#2b6ef5");

        //中下左部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        v.setTitle("温馨提示");
        v.setContent("携带好随身物品").setContentColor("#4e5c74");
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle("验证方式");
        userVO.setVaccineNum(vaccineNum);
        //底部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent("请通行");
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        userVO.setTravelValidationTitle("动态行程卡");
        userVO.setTravelValidationIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setTravelValidationDesc(null);

        return userVO;
    }

}

